/*
 *  soapUI, copyright (C) 2006 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.actions;

import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.AbstractAction;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JTextField;

import com.eviware.soapui.SoapUI;
import com.eviware.soapui.model.settings.Settings;
import com.eviware.soapui.settings.UISettings;
import com.eviware.soapui.support.components.SimpleForm;
import com.eviware.soapui.support.types.StringToStringMap;
import com.eviware.soapui.ui.desktop.DesktopRegistry;
import com.jgoodies.forms.builder.ButtonBarBuilder;
import com.l2fprod.common.swing.JFontChooser;

public class UIPrefs implements Prefs
{
	public static final String ORDER_PROJECTS = "Order Projects";
	public static final String ORDER_TESTCASES = "Order TestCases";
	public static final String ORDER_REQUESTS = "Order Requests";
	public static final String NO_RESIZE_REQUEST_EDITOR = "Disable auto-resize";
	public static final String START_WITH_REQUEST_TABS = "Tabbed request view";
	public static final String AUTO_VALIDATE_REQUEST = "Validate Requests";
	public static final String ABORT_ON_INVALID_REQUEST = "Abort on invalid";
	public static final String AUTO_VALIDATE_RESPONSE = "Validate Responses";
	public static final String CREATE_BACKUP = "Create Backup";
	public static final String BACKUP_FOLDER = "Backup Folder";
	public static final String DESKTOP_TYPE = "Desktop Type";
	public static final String NATIVE_LAF = "Native L&F";
	public static final String ENABLE_GROOVY_LOG_DURING_LOADTEST = "Do not disable Groovy Log";
	
	private JTextField editorFontTextField;
	private SimpleForm editorForm;
	private final String title;
	private JCheckBox abortCheckBox;
	private JCheckBox autoValidateCheckBox;
	private JCheckBox backupCheckBox;
	private JTextField backupFolder;

	public UIPrefs(String title)
	{
		this.title = title;
	}
	
	public String getTitle()
	{
		return title;
	}

	public SimpleForm getForm()
	{
		if( editorForm == null )
		{
			ButtonBarBuilder builder = new ButtonBarBuilder();
			editorFontTextField = new JTextField( 20 );
			editorFontTextField.setEnabled( false );
			builder.addFixed( editorFontTextField );
			builder.addRelatedGap();
			builder.addFixed( new JButton( new AbstractAction("Select Font..")
					{
						public void actionPerformed(ActionEvent e)
						{
							Font font = JFontChooser.showDialog( null, "Select XML Editor Font", 
									Font.decode( editorFontTextField.getText() ));
							
							if( font != null )
								editorFontTextField.setText( font.getFontName() + "-plain-" + font.getSize() );
						}}));
			
			editorForm = new SimpleForm();
			editorForm.addSpace( 5 );
			editorForm.append( "Editor Font", builder.getPanel() );
			editorForm.addSpace( 5 );
			
			editorForm.appendCheckBox( ORDER_PROJECTS, "(orders projects alphabetically in tree)", false );
			editorForm.appendCheckBox( ORDER_TESTCASES, "(orders testcases alphabetically in tree)", false );
			editorForm.appendCheckBox( ORDER_REQUESTS, "(orders requests alphabetically in tree)", false );
			editorForm.appendSeparator();

			editorForm.appendCheckBox( NO_RESIZE_REQUEST_EDITOR, "(disables automatic resizing of request editors)", true );
			editorForm.appendCheckBox( START_WITH_REQUEST_TABS, "(defaults the request editor to the tabbed layout)", true );
			editorForm.appendSeparator();

			autoValidateCheckBox = editorForm.appendCheckBox( AUTO_VALIDATE_REQUEST, "(always validate request messages before they are sent)", true );
			abortCheckBox = editorForm.appendCheckBox( ABORT_ON_INVALID_REQUEST, "(aborts invalid requests)", true );
			editorForm.appendCheckBox( AUTO_VALIDATE_RESPONSE, "(always validate response messages)", true );
			
			autoValidateCheckBox.addActionListener( new ActionListener() {

				public void actionPerformed(ActionEvent e)
				{
					abortCheckBox.setEnabled( autoValidateCheckBox.isSelected() );
				}} );
			
			editorForm.appendSeparator();
			backupCheckBox = editorForm.appendCheckBox( CREATE_BACKUP, "(backup project files before they are saved)", true );
			backupFolder = editorForm.appendTextField( BACKUP_FOLDER, "(folder to backup to, can be both relative or absolut)" );
			backupCheckBox.addActionListener( new ActionListener() {

				public void actionPerformed(ActionEvent e)
				{
					backupFolder.setEnabled( backupCheckBox.isSelected() );
				}} );
			
			if( SoapUI.isStandalone())
			{
				editorForm.appendSeparator();
				editorForm.appendComboBox( DESKTOP_TYPE, DesktopRegistry.getInstance().getNames(), 
							"Select the type of desktop to use" );
				editorForm.appendCheckBox( NATIVE_LAF, "(use native Look & Feel - requires restart)", true );
			}
			
			editorForm.appendSeparator();
			editorForm.appendCheckBox( ENABLE_GROOVY_LOG_DURING_LOADTEST, "(do not disable the groovy log when running LoadTests)", true );
		}
		
		return editorForm;
	}

	public void getFormValues(Settings settings)
	{
		StringToStringMap values = new StringToStringMap();
		editorForm.getValues( values );
		storeValues(values, settings);
	}
	
	public void storeValues(StringToStringMap values, Settings settings)
	{
		if( editorFontTextField != null )
			settings.setString( UISettings.EDITOR_FONT, editorFontTextField.getText() );
		
		settings.setBoolean( UISettings.ORDER_PROJECTS, values.getBoolean( ORDER_PROJECTS ));
		settings.setBoolean( UISettings.ORDER_REQUESTS, values.getBoolean( ORDER_REQUESTS ));
		settings.setBoolean( UISettings.ORDER_TESTCASES, values.getBoolean( ORDER_TESTCASES ));
		settings.setBoolean( UISettings.NO_RESIZE_REQUEST_EDITOR, values.getBoolean( NO_RESIZE_REQUEST_EDITOR ));
		settings.setBoolean( UISettings.START_WITH_REQUEST_TABS, values.getBoolean( START_WITH_REQUEST_TABS ));
		settings.setBoolean( UISettings.AUTO_VALIDATE_REQUEST, values.getBoolean( AUTO_VALIDATE_REQUEST ));
		settings.setBoolean( UISettings.ABORT_ON_INVALID_REQUEST, values.getBoolean( ABORT_ON_INVALID_REQUEST ));
		settings.setBoolean( UISettings.AUTO_VALIDATE_RESPONSE, values.getBoolean( AUTO_VALIDATE_RESPONSE ));
		settings.setBoolean( UISettings.CREATE_BACKUP, values.getBoolean( CREATE_BACKUP ));
		settings.setString( UISettings.BACKUP_FOLDER, values.get( BACKUP_FOLDER ));
		
		if( SoapUI.isStandalone())
		{
			settings.setString( UISettings.DESKTOP_TYPE, values.get( DESKTOP_TYPE ));
			settings.setBoolean( UISettings.NATIVE_LAF, values.getBoolean( NATIVE_LAF ));
		}
		
		settings.setBoolean( UISettings.DONT_DISABLE_GROOVY_LOG, values.getBoolean( ENABLE_GROOVY_LOG_DURING_LOADTEST ));
	}

	public void setFormValues(Settings settings)
	{
		String editorFont = settings.getString( UISettings.EDITOR_FONT, UISettings.DEFAULT_EDITOR_FONT );
		if( editorFont == null || editorFont.length() == 0 )
			editorFont = UISettings.DEFAULT_EDITOR_FONT;
		
		editorFontTextField.setText( editorFont );
		editorForm.setValues( getValues(settings) );
		
		abortCheckBox.setEnabled( settings.getBoolean( UISettings.AUTO_VALIDATE_REQUEST ) );
		backupFolder.setEnabled( settings.getBoolean( UISettings.CREATE_BACKUP ) );
	}

	public StringToStringMap getValues(Settings settings)
	{
		StringToStringMap values = new StringToStringMap();
		values.put( ORDER_PROJECTS, settings.getBoolean( UISettings.ORDER_PROJECTS ));
		values.put( ORDER_REQUESTS, settings.getBoolean( UISettings.ORDER_REQUESTS ));
		values.put( ORDER_TESTCASES, settings.getBoolean( UISettings.ORDER_TESTCASES ));
		values.put( NO_RESIZE_REQUEST_EDITOR, settings.getBoolean( UISettings.NO_RESIZE_REQUEST_EDITOR ));
		values.put( START_WITH_REQUEST_TABS, settings.getBoolean( UISettings.START_WITH_REQUEST_TABS ));
		values.put( AUTO_VALIDATE_REQUEST, settings.getBoolean( UISettings.AUTO_VALIDATE_REQUEST ));
		values.put( ABORT_ON_INVALID_REQUEST, settings.getBoolean( UISettings.ABORT_ON_INVALID_REQUEST ));
		values.put( AUTO_VALIDATE_RESPONSE, settings.getBoolean( UISettings.AUTO_VALIDATE_RESPONSE ));
		values.put( CREATE_BACKUP, settings.getBoolean( UISettings.CREATE_BACKUP ));
		values.put( BACKUP_FOLDER, settings.getString( UISettings.BACKUP_FOLDER, "" ));
		
		if( SoapUI.isStandalone())
		{
			values.put( DESKTOP_TYPE, settings.getString( UISettings.DESKTOP_TYPE, "Windows" ));
			values.put( NATIVE_LAF, settings.getBoolean( UISettings.NATIVE_LAF ));
		}
		
		values.put( ENABLE_GROOVY_LOG_DURING_LOADTEST, settings.getBoolean( UISettings.DONT_DISABLE_GROOVY_LOG ));
		
		return values;
	}
}
