/*
 *  soapUI, copyright (C) 2006 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.actions.mockservice;

import java.awt.event.ActionEvent;
import java.util.ArrayList;
import java.util.List;

import javax.swing.AbstractAction;
import javax.swing.Action;

import com.eviware.soapui.impl.wsdl.WsdlOperation;
import com.eviware.soapui.impl.wsdl.mock.WsdlMockOperation;
import com.eviware.soapui.impl.wsdl.mock.WsdlMockResponse;
import com.eviware.soapui.impl.wsdl.mock.WsdlMockService;
import com.eviware.soapui.model.iface.Interface;
import com.eviware.soapui.support.UISupport;

/**
 * Adds a new WsdlLoadMock to a WsdlMockService
 * 
 * @author Ole.Matzura
 */

public class AddNewMockOperationAction extends AbstractAction
{
   private final WsdlMockService mockService;

	public AddNewMockOperationAction( WsdlMockService mockService ) 
   {
      super( "New MockOperation" );
		this.mockService = mockService;
      putValue( Action.SHORT_DESCRIPTION, "Creates a new MockOperation for this MockService" );
      putValue( Action.ACCELERATOR_KEY, UISupport.getKeyStroke( "menu N" ));
   }
   
   public void actionPerformed(ActionEvent e)
	{
      List<OperationWrapper> operations = new ArrayList<OperationWrapper>();
      
      for( int c = 0; c < mockService.getProject().getInterfaceCount(); c++ )
      {
      	Interface iface = mockService.getProject().getInterfaceAt( c );
      	for( int i = 0; i < iface.getOperationCount(); i++ )
      	{
      		if( !mockService.hasMockOperation( iface.getOperationAt( i  )))
      			operations.add( new OperationWrapper( ( WsdlOperation ) iface.getOperationAt( i )));
      	}
      }
      
      if( operations.isEmpty() )
      {
      	UISupport.showErrorMessage( "No unique operations to mock in project!" );
      	return;
      }
      
      Object result = UISupport.prompt( "Select Operation to Mock", "New MockOperation", operations.toArray() );
      if( result != null )
      {
      	WsdlMockOperation mockOperation = mockService.addNewMockOperation( ((OperationWrapper)result).getOperation());
      	WsdlMockResponse mockResponse = mockOperation.addNewMockResponse( "Response 1", true );
      	UISupport.selectAndShow( mockResponse );
      }
   }
   
   public class OperationWrapper
	{
		private final WsdlOperation operation;

		public OperationWrapper( WsdlOperation operation )
		{
			this.operation = operation;
		}

		public WsdlOperation getOperation()
		{
			return operation;
		}

		public String toString()
		{
			return operation.getInterface().getName() + " - " + operation.getName();
		}
	}
}
