/*
 *  soapUI, copyright (C) 2006 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.actions.project;

import java.awt.event.ActionEvent;
import java.io.File;
import java.io.IOException;

import javax.swing.AbstractAction;
import javax.swing.Action;

import com.eviware.soapui.impl.wsdl.WsdlProject;
import com.eviware.soapui.support.UISupport;

/**
 * Renames a WsdlProject
 * 
 * @author Ole.Matzura
 */

public class SaveProjectAsAction extends AbstractAction
{
   private final WsdlProject project;

	public SaveProjectAsAction( WsdlProject project )
   {
      super( "Save Project As" );
		this.project = project;
      putValue( Action.SHORT_DESCRIPTION, "Saves this project to a new file" );
      putValue( Action.ACCELERATOR_KEY, UISupport.getKeyStroke( "menu shift S" ));
   }
   
   public void actionPerformed(ActionEvent e)
	{
      try
		{
      	File file = UISupport.getFileDialogs().saveAs(this, "Select soapui project file", "xml", "XML", new File( project.getPath()));
         if( file == null ) return;
          
         String fileName = file.getAbsolutePath();
         if( fileName == null ) return;
          
			if( project.saveTo( fileName ))
			{
            project.getWorkspace().save( true );
			}
		}
		catch (IOException e1)
		{
			UISupport.showErrorMessage( "Failed to save project; " + e1 ); 
		}
   }
}
