/*
 *  soapUI, copyright (C) 2006 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.actions.teststep;

import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.Action;

import com.eviware.soapui.SoapUI;
import com.eviware.soapui.impl.wsdl.testcase.WsdlTestCase;
import com.eviware.soapui.impl.wsdl.teststeps.WsdlTestStep;
import com.eviware.soapui.support.UISupport;

/**
 * Deletes a WsdlTestStep from its WsdlTestCase
 * 
 * @author Ole.Matzura
 */

public class DeleteTestStepAction extends AbstractAction
{
   private final WsdlTestStep testStep;

	public DeleteTestStepAction( WsdlTestStep testStep )
   {
      super( "Delete" );
		this.testStep = testStep;
      putValue( Action.SHORT_DESCRIPTION, "Deletes this TestStep" );
      putValue( Action.ACCELERATOR_KEY, UISupport.getKeyStroke( "DELETE" ));
   }
   
   public void actionPerformed(ActionEvent e)
	{
   	if( SoapUI.getTestMonitor().hasRunningTest( testStep.getTestCase() ))
   	{
   		UISupport.showErrorMessage( "Cannot remove step while tests are running" );
   		return;
   	}
   	
      if( UISupport.confirm( "Delete TestStep [" + testStep.getName() + "] from Testcase [" + 
      		testStep.getTestCase().getName() + "]", "Delete TestStep" ))
      {
      	((WsdlTestCase)testStep.getTestCase()).removeTestStep( testStep );
      }
   }

}
