/*
 *  soapUI, copyright (C) 2006 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.panels.mock;

import java.awt.event.ActionEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.JList;
import javax.swing.JPopupMenu;

import com.eviware.soapui.support.UISupport;
import com.eviware.soapui.support.action.ActionList;
import com.eviware.soapui.support.action.ActionSupport;

public abstract class ListMouseListener extends MouseAdapter
{
	protected abstract ActionList getActionsForRow( JList list, int row );

	public void mouseClicked( MouseEvent e )
	{
		if (e.getClickCount() < 2)
			return;
		
		JList list = ( JList ) e.getSource();
		
		int selectedIndex = list.getSelectedIndex();
		if (selectedIndex == -1)
			return;
		
		ActionList actions = getActionsForRow( list, selectedIndex );
		
		if(actions != null)
			actions.performDefaultAction(new ActionEvent(this, 0, null));
	}

	public void mousePressed( MouseEvent e )
	{
		if (e.isPopupTrigger())
			showPopup(e);
	}

	public void mouseReleased( MouseEvent e )
	{
		if (e.isPopupTrigger())
			showPopup(e);
	}

	public void showPopup( MouseEvent e )
	{
		JList list = ( JList ) e.getSource();
		int row = list.locationToIndex(e.getPoint());
		if (row == -1)
			return;
	
		if (list.getSelectedIndex() != row)
		{
			list.setSelectedIndex(row);
		}
	
		ActionList actions = getActionsForRow( list, row );
	
		if (actions == null || actions.getActionCount() == 0)
			return;
	
		JPopupMenu popup = ActionSupport.buildPopup(actions);
		UISupport.showPopup(popup, list, e.getPoint());
	}

}