/*
 *  soapUI, copyright (C) 2006 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.panels.request.components;

import com.eviware.soapui.impl.wsdl.panels.request.components.editor.XmlDocument;
import com.eviware.soapui.impl.wsdl.panels.request.components.editor.XmlEditorView;
import com.eviware.soapui.impl.wsdl.panels.request.components.editor.XmlInspector;
import com.eviware.soapui.impl.wsdl.panels.request.components.editor.inspectors.registry.ResponseInspectorFactory;
import com.eviware.soapui.impl.wsdl.panels.request.components.editor.inspectors.registry.XmlInspectorFactory;
import com.eviware.soapui.impl.wsdl.panels.request.components.editor.inspectors.registry.XmlInspectorRegistry;
import com.eviware.soapui.impl.wsdl.panels.request.components.editor.views.registry.ResponseEditorViewFactory;
import com.eviware.soapui.impl.wsdl.panels.request.components.editor.views.registry.XmlEditorViewFactory;
import com.eviware.soapui.impl.wsdl.panels.request.components.editor.views.registry.XmlEditorViewRegistry;
import com.eviware.soapui.model.ModelItem;

public class ResponseMessageXmlEditor<T extends ModelItem> extends SoapMessageXmlEditor<T>
{
	public ResponseMessageXmlEditor( XmlDocument xmlDocument, T modelItem  )
	{
		super( xmlDocument, modelItem );
		
		XmlEditorViewFactory[] editorFactories = XmlEditorViewRegistry.getInstance().getFactoriesOfType( 
					ResponseEditorViewFactory.class );
		
		for( XmlEditorViewFactory factory : editorFactories )
		{
			ResponseEditorViewFactory f = ( ResponseEditorViewFactory ) factory;
			XmlEditorView editorView = f.createResponseEditorView( this, modelItem );
			if( editorView != null )
				addEditorView( editorView);
		}
		
		XmlInspectorFactory[] inspectorFactories = XmlInspectorRegistry.getInstance().getFactoriesOfType( 
					ResponseInspectorFactory.class );
		
		for( XmlInspectorFactory factory : inspectorFactories )
		{
			ResponseInspectorFactory f = ( ResponseInspectorFactory ) factory;
			XmlInspector inspector = f.createResponseInspector( this, modelItem );
			if( inspector != null )
				addInspector( inspector);
		}
	}
}
