/*
 *  soapUI, copyright (C) 2006 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.panels.request.components.editor.views.registry;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import com.eviware.soapui.impl.wsdl.panels.request.components.editor.views.source.XmlSourceEditorFactory;

public class XmlEditorViewRegistry
{
	private static XmlEditorViewRegistry instance;
	private List<XmlEditorViewFactory> factories = new ArrayList<XmlEditorViewFactory>();
	
	public XmlEditorViewRegistry()
	{
		addFactory( new XmlSourceEditorFactory() );
	}
	
	public void addFactory( XmlEditorViewFactory factory )
	{
		factories.add( factory );
	}

	public void setFactory( String viewId, XmlEditorViewFactory factory )
	{
		for( int c = 0; c < factories.size(); c++ )
		{
			if( factories.get( c ).getViewId().equals( viewId ))
			{
				factories.set( c, factory );
			}
		}
	}
	
	public static final XmlEditorViewRegistry getInstance()
	{
		if( instance == null )
			instance = new XmlEditorViewRegistry();
		
		return instance;
	}
	
	public XmlEditorViewFactory [] getFactories()
	{
		return factories.toArray( new XmlEditorViewFactory[factories.size()] );
	}
	
	public XmlEditorViewFactory [] getFactoriesOfType( Class type )
	{
		List<XmlEditorViewFactory> result = new ArrayList<XmlEditorViewFactory>();
		for( XmlEditorViewFactory factory : factories )
		{
			if( Arrays.asList( factory.getClass().getInterfaces() ).contains( type )) 
				result.add( factory );
		}
		
		return result.toArray( new XmlEditorViewFactory[result.size()] );
	}
}
