/*
 *  soapUI, copyright (C) 2006 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.panels.teststeps;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.Box;
import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;

import org.apache.log4j.Logger;

import com.eviware.soapui.SoapUI;
import com.eviware.soapui.impl.wsdl.actions.support.ShowOnlineHelpAction;
import com.eviware.soapui.impl.wsdl.panels.support.MockTestRunContext;
import com.eviware.soapui.impl.wsdl.panels.support.MockTestRunner;
import com.eviware.soapui.impl.wsdl.panels.support.TestRunComponentEnabler;
import com.eviware.soapui.impl.wsdl.panels.teststeps.support.GroovyEditor;
import com.eviware.soapui.impl.wsdl.panels.teststeps.support.GroovyEditorModel;
import com.eviware.soapui.impl.wsdl.support.HelpUrls;
import com.eviware.soapui.impl.wsdl.teststeps.WsdlGroovyScriptTestStep;
import com.eviware.soapui.impl.wsdl.teststeps.WsdlTestStepResult;
import com.eviware.soapui.model.ModelItem;
import com.eviware.soapui.model.settings.Settings;
import com.eviware.soapui.model.settings.SettingsListener;
import com.eviware.soapui.support.UISupport;
import com.eviware.soapui.support.components.JEditorStatusBarWithProgress;
import com.eviware.soapui.support.components.JXToolBar;
import com.eviware.soapui.support.log.JLogList;
import com.eviware.soapui.ui.desktop.DesktopPanel;

/**
 * DesktopPanel for WsdlGroovyTestSteps
 * 
 * @author Ole.Matzura
 */

public class GroovyScriptStepDesktopPanel extends JPanel implements DesktopPanel
{
	private final WsdlGroovyScriptTestStep groovyStep;
	private GroovyEditor editor;
	private JSplitPane mainSplit;
	private JLogList logArea;
	private Logger logger;
	private TestRunComponentEnabler componentEnabler;
	private RunAction runAction = new RunAction();
	private JEditorStatusBarWithProgress statusBar;
	private SettingsListener settingsListener;

	public GroovyScriptStepDesktopPanel(WsdlGroovyScriptTestStep groovyStep)
	{
		super( new BorderLayout() );
		this.groovyStep = groovyStep;
		componentEnabler = new TestRunComponentEnabler( groovyStep.getTestCase() );
		
		buildUI();
		setPreferredSize( new Dimension( 600, 440 ));
		
		logger = Logger.getLogger( groovyStep.getName() );
	}

	private void buildUI()
	{
		editor = new GroovyEditor( new ScriptStepGroovyEditorModel( ));
		
		logArea = new JLogList( "Groovy Test Log" );
		logArea.addLogger( groovyStep.getName(), true );
		logArea.getLogList().addMouseListener( new MouseAdapter() {

			public void mouseClicked(MouseEvent e)
			{
				if( e.getClickCount() < 2 )
					return;
				
				String value = logArea.getLogList().getSelectedValue().toString();
				if( value == null )
					return;
				
				editor.selectError( value );
			}} );
		
		mainSplit = UISupport.createVerticalSplit( new JScrollPane( editor ), logArea);
		mainSplit.setDividerLocation( 280 );
		mainSplit.setResizeWeight( 0.8 );
		add( mainSplit, BorderLayout.CENTER );
		add( buildToolbar(), BorderLayout.NORTH );
		add( buildStatusBar(), BorderLayout.SOUTH );
		
		componentEnabler.add( editor );
	}
	
	private Component buildStatusBar()
	{
		statusBar = new JEditorStatusBarWithProgress( editor );
		return statusBar;
	}

	private JComponent buildToolbar()
	{
		JXToolBar toolBar = UISupport.createToolbar();
		JButton runButton = UISupport.createToolbarButton( runAction );
		toolBar.add( runButton );
		toolBar.add( Box.createHorizontalGlue() );
		JLabel label = new JLabel("<html>Script is invoked with <code>log</code>, <code>context</code> " +
						"and <code>testRunner</code> variables</html>");
		label.setToolTipText( label.getText() );
		label.setMaximumSize( label.getPreferredSize() );
		
		toolBar.add( label);
		toolBar.addRelatedGap();
		toolBar.add( UISupport.createToolbarButton( new ShowOnlineHelpAction( HelpUrls.GROOVYSTEPEDITOR_HELP_URL )));
		
		componentEnabler.add( runButton );
		
		return toolBar;
	}

	public ModelItem getModelItem()
	{
		return groovyStep;
	}

	public boolean onClose( boolean canCancel )
	{
		componentEnabler.release();
		SoapUI.getSettings().removeSettingsListener( settingsListener );
		return true;
	}

	public JComponent getComponent()
	{
		return this;
	}

	public boolean dependsOn(ModelItem modelItem)
	{
		return modelItem == groovyStep || modelItem == groovyStep.getTestCase() ||
				modelItem == groovyStep.getTestCase().getTestSuite() ||
				modelItem == groovyStep.getTestCase().getTestSuite().getProject();
	}

	public String getTitle()
	{
		return groovyStep.getTestCase().getName() + " - " + groovyStep.getName();
	}
	
	public String getDescription()
	{
		return "Goto: [" + groovyStep.getName() + "] - " + groovyStep.getTestStepTitle();
	}

	public Icon getIcon()
	{
		return getModelItem().getIcon();
	}
	
	private class ScriptStepGroovyEditorModel implements GroovyEditorModel
	{
		public String[] getKeywords()
		{
			return new String[] {"log", "context", "testRunner"};
		}

		public Action getRunAction()
		{
			return runAction;
		}

		public String getScript()
		{
			return groovyStep.getScript();
		}

		public void setScript( String text )
		{
			groovyStep.setScript( text );
		}

		public Settings getSettings()
		{
			return SoapUI.getSettings();
		}}
	
	private class RunAction extends AbstractAction
	{
		public RunAction()
		{
			putValue( Action.SMALL_ICON, UISupport.createImageIcon( "/run_groovy_script.gif" ));
			putValue( Action.SHORT_DESCRIPTION, "Runs this script using a mock testRunner and testContext" );
		}

		public void actionPerformed(ActionEvent e)
		{
			MockTestRunner mockTestRunner = new MockTestRunner( groovyStep.getTestCase(), logger );
			statusBar.setIndeterminate( true );
			WsdlTestStepResult result = (WsdlTestStepResult) groovyStep.run( mockTestRunner, 
					new MockTestRunContext( mockTestRunner, groovyStep ) );
			statusBar.setIndeterminate( false );
			
			if( result.getError() != null )
			{
				String message = result.getError().getMessage();
				
				// ugly...
				editor.selectError( message);
				
				UISupport.showErrorMessage( result.getError().toString() );
				editor.requestFocus();
			}
		}
	}
}
