/*
 *  soapUI, copyright (C) 2006 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.panels.teststeps.support;

import java.awt.BorderLayout;
import java.awt.Font;
import java.awt.event.ActionListener;

import javax.swing.JPanel;
import javax.swing.event.CaretListener;
import javax.swing.text.Document;

import org.syntax.jedit.KeywordMap;
import org.syntax.jedit.tokenmarker.CTokenMarker;
import org.syntax.jedit.tokenmarker.GroovyTokenMarker;
import org.syntax.jedit.tokenmarker.Token;

import com.eviware.soapui.model.settings.Settings;
import com.eviware.soapui.model.settings.SettingsListener;
import com.eviware.soapui.settings.UISettings;
import com.eviware.soapui.support.DocumentListenerAdapter;
import com.eviware.soapui.support.components.JEditorStatusBar.JEditorStatusBarTarget;
import com.eviware.soapui.support.xml.JXEditTextArea;

public class GroovyEditor extends JPanel implements JEditorStatusBarTarget
{
	private JXEditTextArea editArea;
	private final GroovyEditorModel model;
	private InternalSettingsListener settingsListener;

	public GroovyEditor( GroovyEditorModel model )
	{
		super( new BorderLayout() );
		this.model = model;
		
		editArea = new JXEditTextArea( new CTokenMarker( false, initKeywords() ) );
		
		Settings settings = model.getSettings();
		String editorFont = settings.getString( UISettings.EDITOR_FONT, UISettings.DEFAULT_EDITOR_FONT );
		if( editorFont != null && editorFont.length() > 0 )
			editArea.setFont(Font.decode(editorFont));
		else
			editArea.setFont( Font.decode( UISettings.DEFAULT_EDITOR_FONT ));

		editArea.setText( model.getScript() );
		ActionListener runAction = model.getRunAction();
		if( runAction != null )
			editArea.getInputHandler().addKeyBinding( "A+ENTER", runAction );
		
		editArea.getDocument().addDocumentListener( new DocumentListenerAdapter() {

			public void update(Document document)
			{
				GroovyEditor.this.model.setScript( editArea.getText() );
			}} );
		
		settingsListener = new InternalSettingsListener();
		settings.addSettingsListener( settingsListener );
		
		add( editArea );
	}
	
	public JXEditTextArea getEditArea()
	{
		return editArea;
	}

	public void release()
	{
		model.getSettings().removeSettingsListener( settingsListener );
	}

	public void selectError(String message)
	{
		int ix = message == null ? -1 : message.indexOf( "@ line " );
		if( ix >= 0 )
		{
			try
			{
				int ix2 = message.indexOf(',', ix);
				int line = ix2 == -1 ? Integer.parseInt(message.substring(ix + 6).trim()) : Integer.parseInt(message
						.substring(ix + 6, ix2).trim());
				int column = 0;
				if (ix2 != -1)
				{
					ix = message.indexOf("column ", ix2);
					if (ix >= 0)
					{
						ix2 = message.indexOf('.', ix);
						column = ix2 == -1 ? Integer.parseInt(message.substring(ix + 7).trim()) : Integer
								.parseInt(message.substring(ix + 7, ix2).trim());
					}
				}
				
				editArea.setCaretPosition(editArea.getLineStartOffset(line - 1) + column - 1);
			}
			catch (Exception ex)
			{
			}					
			
			editArea.requestFocus();
		}
	}
	
	private KeywordMap initKeywords()
	{
		KeywordMap keywords = GroovyTokenMarker.getKeywords();
		
		for( String keyword : model.getKeywords() )
			keywords.add(keyword,Token.KEYWORD2);
		return keywords;
	}
	
	private final class InternalSettingsListener implements SettingsListener
	{
		public void settingChanged(String name, String newValue, String oldValue)
		{
			if( name.equals( UISettings.EDITOR_FONT ))
			{
				editArea.setFont( Font.decode( newValue ));
				invalidate();
			}
		}
	}

	public void addCaretListener( CaretListener listener )
	{
		editArea.addCaretListener( listener );
	}

	public int getCaretPosition()
	{
		return editArea.getCaretPosition();
	}

	public int getLineOfOffset( int offset ) throws Exception
	{
		return editArea.getLineOfOffset( offset );
	}

	public int getLineStartOffset( int line ) throws Exception
	{
		return editArea.getLineStartOffset( line );
	}

	public void removeCaretListener( CaretListener listener )
	{
		editArea.removeCaretListener( listener );
	}
}
