/*
 *  soapUI, copyright (C) 2006 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.submit.transports.http;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.MalformedURLException;

import javax.mail.BodyPart;
import javax.mail.MessagingException;

import com.eviware.soapui.model.iface.Attachment;
import com.eviware.soapui.support.Tools;

public class BodyPartAttachment implements Attachment
{
	private final BodyPart bodyPart;
	private File tempFile;

	public BodyPartAttachment(BodyPart bodyPart)
	{
		this.bodyPart = bodyPart;
	}

	public String getContentType()
	{
		try
		{
			return bodyPart.getContentType();
		}
		catch (MessagingException e)
		{
			e.printStackTrace();
			return null;
		}
	}

	public InputStream getInputStream() throws IOException
	{
		try
		{
			return bodyPart.getInputStream();
		}
		catch (MessagingException e)
		{
			e.printStackTrace();
			return null;
		}
	}

	public String getName()
	{
		try
		{
			String[] header = bodyPart.getHeader( "Content-Id" );
			if( header == null || header.length == 0 )
				return "<missing name>";
			
			if( header[0].startsWith( "<" ) && header[0].endsWith( ">" ))
				header[0] = header[0].substring( 1, header[0].length()-1 );
			
			return header[0];
		}
		catch (MessagingException e)
		{
			e.printStackTrace();
			return null;
		}
	}

	public String getPart()
	{
		return getName();
	}

	public long getSize()
	{
		try
		{
			return bodyPart.getSize();
		}
		catch (MessagingException e)
		{
			e.printStackTrace();
			return -1;
		}
	}

	public String getUrl()
	{
		if( tempFile == null )
		{
			String contentType = getContentType();
			int ix = contentType.lastIndexOf( '/' );
			int iy = -1;
			if (ix != -1)
			iy = contentType.indexOf(';', ix);
	
			try
			{
				tempFile = File.createTempFile( "response-attachment", (ix == -1 ? ".dat" : "." + (iy == -1 ?
					contentType.substring(ix+1) : contentType.substring(ix+1, iy))));

				OutputStream out = new BufferedOutputStream( new FileOutputStream( tempFile ));
				InputStream inputStream = getInputStream();
				out.write( Tools.readAll( inputStream, 0 ).toByteArray() );
				out.flush();
				out.close();
	
				inputStream.reset();
			}
			catch (Exception e)
			{
				e.printStackTrace();
			}
		}
			
		try
		{
			return tempFile.toURL().toString();
		}
		catch (MalformedURLException e)
		{
			e.printStackTrace();
			return null;
		}
	}

	public void setContentType(String contentType)
	{
	}

	public void setPart(String part)
	{
	}

	public boolean isCached()
	{
		return true;
	}

	public AttachmentType getAttachmentType()
	{
		return AttachmentType.UNKNOWN;
	}

	public void release()
	{
	}

	public String getContentID()
	{
		try
		{
			String[] header = bodyPart.getHeader( "Content-ID" );
			if( header != null && header.length > 0 )
				return header[0];
		}
		catch( MessagingException e )
		{
			e.printStackTrace();
		}
		
		return null;
	}
}
