/*
 *  soapUI, copyright (C) 2006 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */
 
package com.eviware.soapui.impl.wsdl.submit.transports.http;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import javax.activation.DataSource;

import com.eviware.soapui.support.Tools;

public class PostResponseDataSource implements DataSource
{
	private final TimeablePostMethod postMethod;
	private byte[] data;

	public PostResponseDataSource(TimeablePostMethod postMethod)
	{
		this.postMethod = postMethod;
		
		try
		{
			data = Tools.readAll( postMethod.getResponseBodyAsStream(), 0 ).toByteArray();
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}		
	}
	
	public long getDataSize()
	{
		return data == null ? -1 : data.length;
	}

	public String getContentType()
	{
		return postMethod.getResponseHeader( "Content-Type" ).getValue();
	}

	public InputStream getInputStream() throws IOException
	{
		return new ByteArrayInputStream( data );
	}

	public String getName()
	{
		return postMethod.getName() + " response for " + postMethod.getPath().toString();
	}

	public OutputStream getOutputStream() throws IOException
	{
		return null;
	}

}
