/*
 *  soapUI, copyright (C) 2006 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.teststeps.support;

import java.io.PrintWriter;
import java.lang.ref.SoftReference;

import com.eviware.soapui.impl.wsdl.submit.WsdlMessageExchange;
import com.eviware.soapui.impl.wsdl.teststeps.WsdlTestStep;
import com.eviware.soapui.impl.wsdl.teststeps.WsdlTestStepResult;
import com.eviware.soapui.impl.wsdl.teststeps.actions.ShowMessageExchangeAction;
import com.eviware.soapui.support.types.StringToStringMap;
import com.eviware.soapui.support.xml.XmlUtils;

public class WsdlMessageExchangeStepResult extends WsdlTestStepResult
{
	private SoftReference<WsdlMessageExchange> messageExchange;
	private SoftReference<StringToStringMap> properties;
	
	public WsdlMessageExchangeStepResult(WsdlTestStep step )
	{
		super( step );
	}
	
	public void setMessageExchange( WsdlMessageExchange messageExchange )
	{
		this.messageExchange = new SoftReference<WsdlMessageExchange>( messageExchange );
		addAction( new ShowMessageExchangeAction( messageExchange, "StepResult" ), true );
	}

	public String getRequestContent()
	{
		if( isDiscarded() )
			return "<discarded>";
		
		if( messageExchange != null && messageExchange.get() == null ) 
			return "<garbage-collected>";
		
		return messageExchange == null ? null : messageExchange.get().getRequestContent();
	}

	public void addProperty( String name, String value )
	{
		if( isDiscarded() )
			return;
		
		if( properties == null )
			properties = new SoftReference<StringToStringMap>( new StringToStringMap() );
		
		properties.get().put( name, value );
	}
	
	public void discard()
	{
		super.discard();
		
		messageExchange = null;
		properties = null;
	}

	public void writeTo(PrintWriter writer)
	{
		super.writeTo( writer );
		
		if( isDiscarded() )
			return;
		
		writer.println( "---------------- Properties ------------------------" );
		StringToStringMap map = properties == null ? null : properties.get();
		if( map == null || map.isEmpty() )
		{
			writer.println( "Missing Properties" );
		}
		else
		{
			for( String name : map.keySet() )
				writer.println( name + ": " + map.get( name ) );
		}

		writer.println( "---------------- Message Exchange ------------------" );
		WsdlMessageExchange me = messageExchange == null ? null : messageExchange.get();
		if( me == null )
		{
			writer.println( "Missing MessageExchange" );
		}
		else
		{
			writer.println( "--- Request" );
			if( me.getRequestHeaders() != null )
				writer.println( "Request Headers: " + me.getRequestHeaders().toString() );
			
			writer.println( XmlUtils.prettyPrintXml( me.getRequestContent() ) );

			writer.println( "--- Response" );
			if( me.getResponseHeaders() != null )
				writer.println( "Response Headers: " + me.getResponseHeaders().toString() );
			
			writer.println( XmlUtils.prettyPrintXml( me.getResponseContent() ) );
		}
	}

	public StringToStringMap getProperties()
	{
		return properties == null ? null : properties.get();
	}
}