/*
 *  soapUI, copyright (C) 2006 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.ui.support;

import java.awt.BorderLayout;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;

import javax.swing.Action;
import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JPanel;

import com.eviware.soapui.model.ModelItem;
import com.eviware.soapui.support.UISupport;
import com.eviware.soapui.ui.desktop.DesktopPanel;

/**
 * Base class for DesktopPanels..
 */

public abstract class ModelItemDesktopPanel<T extends ModelItem> extends JPanel implements DesktopPanel
{
	private PropertyChangeSupport propertyChangeSupport;
	private final T modelItem;
	private InternalPropertyChangeListener propertyChangeListener;

	public ModelItemDesktopPanel( T modelItem )
	{
		super( new BorderLayout() );
		this.modelItem = modelItem;
		propertyChangeSupport = new PropertyChangeSupport( this );
		
		propertyChangeListener = new InternalPropertyChangeListener();
		((ModelItem)modelItem).addPropertyChangeListener( propertyChangeListener );
	}

	protected boolean release()
	{
		modelItem.removePropertyChangeListener( propertyChangeListener );
		return true;
	}
	
	public JComponent getComponent()
	{
		return this;
	}
	
	final public T getModelItem()
	{
		return modelItem;
	}

	public Icon getIcon()
	{
		return modelItem.getIcon();
	}

	public boolean dependsOn(ModelItem modelItem)
	{
		return false;
	}
	
	public String getTitle()
	{
		return modelItem.getName();
	}
	
	public String getDescription()
	{
		return modelItem.getDescription();
	}

	protected JButton createActionButton(Action action, boolean enabled)
	{
		JButton button = UISupport.createToolbarButton(action, enabled);
		action.putValue(Action.NAME, null);
		return button;
	}

	public void addPropertyChangeListener(String propertyName,
			PropertyChangeListener listener)
	{
		propertyChangeSupport.addPropertyChangeListener( propertyName, listener );
	}

	public void addPropertyChangeListener(PropertyChangeListener listener)
	{
		propertyChangeSupport.addPropertyChangeListener( listener );
	}

	public void removePropertyChangeListener(PropertyChangeListener listener)
	{
		propertyChangeSupport.removePropertyChangeListener( listener );
	}

	public void removePropertyChangeListener(String propertyName,
			PropertyChangeListener listener)
	{
		propertyChangeSupport.removePropertyChangeListener( propertyName, listener );
	}
	
	public void notifyPropertyChange( String propertyName, Object oldValue, Object newValue )
	{
		propertyChangeSupport.firePropertyChange( propertyName, oldValue, newValue );
	}
	
	private class InternalPropertyChangeListener implements PropertyChangeListener
	{
		public void propertyChange(PropertyChangeEvent evt)
		{
			if( evt.getPropertyName().equals( ModelItem.NAME_PROPERTY ))
				notifyPropertyChange( DesktopPanel.TITLE_PROPERTY, null, getTitle() );
			
			if( evt.getPropertyName().equals( ModelItem.ICON_PROPERTY ))
				notifyPropertyChange( DesktopPanel.ICON_PROPERTY, null, getIcon() );
		}
	}
}
